@@ -1,4 +1,5 @@
 #include "duckdb/function/scalar/sequence_functions.hpp"
+#include "duckdb/function/scalar/sequence_utils.hpp"
 
 #include "duckdb/catalog/catalog.hpp"
 #include "duckdb/catalog/dependency_list.hpp"
@@ -16,6 +17,8 @@
 
 namespace duckdb {
 
+namespace {
+
 struct CurrentSequenceValueOperator {
 	static int64_t Operation(DuckTransaction &, SequenceCatalogEntry &seq) {
 		return seq.CurrentValue();
@@ -28,15 +31,22 @@ struct NextSequenceValueOperator {
 	}
 };
 
-SequenceCatalogEntry &BindSequence(ClientContext &context, string &catalog, string &schema, const string &name) {
+SequenceCatalogEntry &BindSequence(Binder &binder, string &catalog, string &schema, const string &name) {
 	// fetch the sequence from the catalog
+	Binder::BindSchemaOrCatalog(binder.context, catalog, schema);
+	EntryLookupInfo sequence_lookup(CatalogType::SEQUENCE_ENTRY, name);
+	return binder.EntryRetriever().GetEntry(catalog, schema, sequence_lookup)->Cast<SequenceCatalogEntry>();
+}
+
+SequenceCatalogEntry &BindSequenceFromContext(ClientContext &context, string &catalog, string &schema,
+                                              const string &name) {
 	Binder::BindSchemaOrCatalog(context, catalog, schema);
 	return Catalog::GetEntry<SequenceCatalogEntry>(context, catalog, schema, name);
 }
 
-SequenceCatalogEntry &BindSequence(ClientContext &context, const string &name) {
+SequenceCatalogEntry &BindSequence(Binder &binder, const string &name) {
 	auto qname = QualifiedName::Parse(name);
-	return BindSequence(context, qname.catalog, qname.schema, qname.name);
+	return BindSequence(binder, qname.catalog, qname.schema, qname.name);
 }
 
 struct NextValLocalState : public FunctionLocalState {
@@ -61,7 +71,7 @@ unique_ptr<FunctionLocalState> NextValLocalFunction(ExpressionState &state, cons
 }
 
 template <class OP>
-static void NextValFunction(DataChunk &args, ExpressionState &state, Vector &result) {
+void NextValFunction(DataChunk &args, ExpressionState &state, Vector &result) {
 	auto &func_expr = state.expr.Cast<BoundFunctionExpression>();
 	if (!func_expr.bind_info) {
 		// no bind info - return null
@@ -80,30 +90,26 @@ static void NextValFunction(DataChunk &args, ExpressionState &state, Vector &res
 	}
 }
 
-static unique_ptr<FunctionData> NextValBind(ClientContext &context, ScalarFunction &,
-                                            vector<unique_ptr<Expression>> &arguments) {
+unique_ptr<FunctionData> NextValBind(ScalarFunctionBindInput &bind_input, ScalarFunction &,
+                                     vector<unique_ptr<Expression>> &arguments) {
+	if (arguments[0]->HasParameter() || arguments[0]->return_type.id() == LogicalTypeId::UNKNOWN) {
+		throw ParameterNotResolvedException();
+	}
 	if (!arguments[0]->IsFoldable()) {
 		throw NotImplementedException(
 		    "currval/nextval requires a constant sequence - non-constant sequences are no longer supported");
 	}
+	auto &binder = bind_input.binder;
 	// parameter to nextval function is a foldable constant
 	// evaluate the constant and perform the catalog lookup already
-	auto seqname = ExpressionExecutor::EvaluateScalar(context, *arguments[0]);
+	auto seqname = ExpressionExecutor::EvaluateScalar(binder.context, *arguments[0]);
 	if (seqname.IsNull()) {
 		return nullptr;
 	}
-	auto &seq = BindSequence(context, seqname.ToString());
+	auto &seq = BindSequence(binder, seqname.ToString());
 	return make_uniq<NextvalBindData>(seq);
 }
 
-static void NextValDependency(BoundFunctionExpression &expr, LogicalDependencyList &dependencies) {
-	if (!expr.bind_info) {
-		return;
-	}
-	auto &info = expr.bind_info->Cast<NextvalBindData>();
-	dependencies.AddDependency(info.sequence);
-}
-
 void Serialize(Serializer &serializer, const optional_ptr<FunctionData> bind_data, const ScalarFunction &) {
 	auto &next_val_bind_data = bind_data->Cast<NextvalBindData>();
 	serializer.WritePropertyWithDefault(100, "sequence_create_info", next_val_bind_data.create_info);
@@ -117,7 +123,7 @@ unique_ptr<FunctionData> Deserialize(Deserializer &deserializer, ScalarFunction
 	}
 	auto &seq_info = create_info->Cast<CreateSequenceInfo>();
 	auto &context = deserializer.Get<ClientContext &>();
-	auto &sequence = BindSequence(context, seq_info.catalog, seq_info.schema, seq_info.name);
+	auto &sequence = BindSequenceFromContext(context, seq_info.catalog, seq_info.schema, seq_info.name);
 	return make_uniq<NextvalBindData>(sequence);
 }
 
@@ -129,25 +135,31 @@ void NextValModifiedDatabases(ClientContext &context, FunctionModifiedDatabasesI
 	input.properties.RegisterDBModify(seq.sequence.ParentCatalog(), context);
 }
 
-void NextvalFun::RegisterFunction(BuiltinFunctions &set) {
+} // namespace
+
+ScalarFunction NextvalFun::GetFunction() {
 	ScalarFunction next_val("nextval", {LogicalType::VARCHAR}, LogicalType::BIGINT,
-	                        NextValFunction<NextSequenceValueOperator>, NextValBind, NextValDependency);
+	                        NextValFunction<NextSequenceValueOperator>, nullptr, nullptr);
+	next_val.bind_extended = NextValBind;
 	next_val.stability = FunctionStability::VOLATILE;
 	next_val.serialize = Serialize;
 	next_val.deserialize = Deserialize;
 	next_val.get_modified_databases = NextValModifiedDatabases;
 	next_val.init_local_state = NextValLocalFunction;
-	set.AddFunction(next_val);
+	BaseScalarFunction::SetReturnsError(next_val);
+	return next_val;
 }
 
-void CurrvalFun::RegisterFunction(BuiltinFunctions &set) {
+ScalarFunction CurrvalFun::GetFunction() {
 	ScalarFunction curr_val("currval", {LogicalType::VARCHAR}, LogicalType::BIGINT,
-	                        NextValFunction<CurrentSequenceValueOperator>, NextValBind, NextValDependency);
+	                        NextValFunction<CurrentSequenceValueOperator>, nullptr, nullptr);
+	curr_val.bind_extended = NextValBind;
 	curr_val.stability = FunctionStability::VOLATILE;
 	curr_val.serialize = Serialize;
 	curr_val.deserialize = Deserialize;
 	curr_val.init_local_state = NextValLocalFunction;
-	set.AddFunction(curr_val);
+	BaseScalarFunction::SetReturnsError(curr_val);
+	return curr_val;
 }
 
 } // namespace duckdb
