@@ -8,15 +8,27 @@
 
 namespace duckdb {
 
-void ArrowTableType::AddColumn(idx_t index, shared_ptr<ArrowType> type) {
+void ArrowTableSchema::AddColumn(idx_t index, shared_ptr<ArrowType> type, const string &name) {
 	D_ASSERT(arrow_convert_data.find(index) == arrow_convert_data.end());
+	if (index >= types.size()) {
+		types.resize(index + 1);
+		column_names.resize(index + 1);
+	}
+	types[index] = type->GetDuckType(true);
+	column_names[index] = name;
 	arrow_convert_data.emplace(std::make_pair(index, std::move(type)));
 }
 
-const arrow_column_map_t &ArrowTableType::GetColumns() const {
+const arrow_column_map_t &ArrowTableSchema::GetColumns() const {
 	return arrow_convert_data;
 }
+vector<LogicalType> &ArrowTableSchema::GetTypes() {
+	return types;
+}
 
+vector<string> &ArrowTableSchema::GetNames() {
+	return column_names;
+}
 void ArrowType::SetDictionary(unique_ptr<ArrowType> dictionary) {
 	D_ASSERT(!this->dictionary_type);
 	dictionary_type = std::move(dictionary);
@@ -105,7 +117,22 @@ unique_ptr<ArrowType> ArrowType::GetTypeFromFormat(string &format) {
 		if (width > 38 || bitwidth > 128) {
 			throw NotImplementedException("Unsupported Internal Arrow Type for Decimal %s", format);
 		}
-		return make_uniq<ArrowType>(LogicalType::DECIMAL(NumericCast<uint8_t>(width), NumericCast<uint8_t>(scale)));
+		switch (bitwidth) {
+		case 32:
+			return make_uniq<ArrowType>(LogicalType::DECIMAL(NumericCast<uint8_t>(width), NumericCast<uint8_t>(scale)),
+			                            make_uniq<ArrowDecimalInfo>(DecimalBitWidth::DECIMAL_32));
+		case 64:
+			return make_uniq<ArrowType>(LogicalType::DECIMAL(NumericCast<uint8_t>(width), NumericCast<uint8_t>(scale)),
+			                            make_uniq<ArrowDecimalInfo>(DecimalBitWidth::DECIMAL_64));
+		case 128:
+			return make_uniq<ArrowType>(LogicalType::DECIMAL(NumericCast<uint8_t>(width), NumericCast<uint8_t>(scale)),
+			                            make_uniq<ArrowDecimalInfo>(DecimalBitWidth::DECIMAL_128));
+		case 256:
+			return make_uniq<ArrowType>(LogicalType::DECIMAL(NumericCast<uint8_t>(width), NumericCast<uint8_t>(scale)),
+			                            make_uniq<ArrowDecimalInfo>(DecimalBitWidth::DECIMAL_256));
+		default:
+			throw NotImplementedException("Unsupported bit-width value of %d  for Arrow Decimal type", bitwidth);
+		}
 	} else if (format == "u") {
 		return make_uniq<ArrowType>(LogicalType::VARCHAR, make_uniq<ArrowStringInfo>(ArrowVariableSizeType::NORMAL));
 	} else if (format == "U") {
@@ -157,6 +184,9 @@ unique_ptr<ArrowType> ArrowType::GetTypeFromFormat(string &format) {
 	} else if (format == "Z") {
 		auto type_info = make_uniq<ArrowStringInfo>(ArrowVariableSizeType::SUPER_SIZE);
 		return make_uniq<ArrowType>(LogicalType::BLOB, std::move(type_info));
+	} else if (format == "vz") {
+		auto type_info = make_uniq<ArrowStringInfo>(ArrowVariableSizeType::VIEW);
+		return make_uniq<ArrowType>(LogicalType::BLOB, std::move(type_info));
 	} else if (format[0] == 'w') {
 		string parameters = format.substr(format.find(':') + 1);
 		auto fixed_size = NumericCast<idx_t>(std::stoi(parameters));
@@ -362,6 +392,16 @@ bool ArrowType::HasExtension() const {
 	return extension_data.get() != nullptr;
 }
 
+ArrowArrayPhysicalType ArrowType::GetPhysicalType() const {
+	if (HasDictionary()) {
+		return ArrowArrayPhysicalType::DICTIONARY_ENCODED;
+	}
+	if (RunEndEncoded()) {
+		return ArrowArrayPhysicalType::RUN_END_ENCODED;
+	}
+	return ArrowArrayPhysicalType::DEFAULT;
+}
+
 unique_ptr<ArrowType> ArrowType::GetTypeFromSchema(DBConfig &config, ArrowSchema &schema) {
 	auto format = string(schema.format);
 	// Let's first figure out if this type is an extension type
